classdef simSetup < handle
    properties
        Figure;
        simulation;
        savedSim;
    end
    
    methods
        %{
        simSetup
        ---------------------------------------------------------------------------
        Description:
            Constructor for simSetup
            Calls the runSimulation function.
        Inputs:
            xmlInput - a structure containing simulation parameters
        Outputs:
            app - a simSetup instance
        %}
        function app = simSetup(xmlInput)
            %{
            app.Figure = figure('MenuBar','none', 'NumberTitle','off',...
                'Name','Simulation Builder', 'CloseRequestFcn',@app.closeApp);
            app.runBtn = uicontrol(app.Figure, 'Style','pushbutton',...
                'Units','normalized','Position',[0.01 0.94 0.1 0.05],...
                'String','Start', 'Callback', @app.runSimulation);
            %}
            app.runSimulation(xmlInput);
        end
        
        %{
        closeApp
        ---------------------------------------------------------------------------
        Description:
            Closes the simulation and its interface
        Inputs:
            app - a simSetup instance
        Outputs:
            NA
        %}
        function closeApp(app,hObject,eventdata)
            delete(app.simulation);
            delete(app.Figure);
        end
        
        %{
        resetSimulation
        ---------------------------------------------------------------------------
        Description:
            Creates a new aSimulationSpace instance that can run another
            simulation.
        Inputs:
            app - a simSetup instance
            p - a structure containing simulation parameters
        Outputs:
            sim - aSimulationSpace instance
        %}
        function sim = resetSimulation(app,p,hObject,eventdata)
            disp('resetting');
            sim = aSimulationSpace(...
            'tick_time', p.ticktime,...
            'x_bound', p.xbound,...
            'y_bound', p.ybound,...
            'z_bound', p.zbound,...
            'num_satellite_cells', p.numsatellitecells,...
            'satellite_cell_speed', p.satellitecellspeed,...
            'myoblast_spawn_chance', p.myoblastspawnchance,...
            'satellite_cell_die_chance', p.satellitecelldiechance,...
            'satellite_cell_radius', p.satellitecellradius,...
            'satellite_cell_world_restriction', p.satellitecellworldrestriction,...
            'num_myoblasts', p.nummyoblasts,...
            'myoblast_speed', p.myoblastspeed,...
            'myoblast_die_chance', p.myoblastdiechance,...
            'myoblast_radius', p.myoblastradius,...
            'myoblast_world_restriction', p.myoblastworldrestriction,...
            'num_nuclei', p.numnuclei,...
            'max_muscle_fibers', p.maxmusclefibers,...
            'num_muscle_fibers', p.nummusclefibers,...
            'max_muscle_fibers', p.maxmusclefibers,...
            'fibril_radius', p.fibrilradius,...
            'fibril_length', p.fibrillength,...
            'fibril_grow_rate', p.fibrilgrowrate,...
            'fibril_decay_rate', p.fibrildecayrate,...
            'num_myofibrils', p.nummyofibrils,...
            'myofibril_max_radius', p.myofibrilmaxradius,...
            'myofibril_max_length', p.myofibrilmaxlength,...
            'myofibril_max_growth_rate', p.myofibrilmaxgrowthrate,...
            'myofibril_max_decay_rate', p.myofibrilmaxdecayrate,...
            'interfibril_distance', p.interfibrildistance,...
            'myofibril_edge_distance', p.myofibriledgedistance,...
            'myofibril_projection_quality', p.myofibrilprojectionquality,...
            'nucleus_speed', p.nucleusspeed,...
            'nucleus_die_chance', p.nucleusdiechance,...
            'nucleus_radius', p.nucleusradius,... 
            'nucleus_random_spread', p.nucleusrandomspread,...
            'nucleus_push_spread', p.nucleuspushspread,...
            'myoblast_random_spread', p.myoblastrandomspread,...
            'myoblast_pull_spread', p.myoblastpullspread,...
            'myoblast_muscle_pull_spread', p.myoblastmusclepullspread,...
            'myofibril_grow_num_nucleus', p.myofibrilgrownumnucleus,...
            'myofibril_grow_dist_nucleus', p.myofibrilgrowdistnucleus,...
            'myofibril_grow_nuclear_density', p.myofibrilgrownucleardensity,...
            'myofibril_decay_size_myofibril', p.myofibrildecaysizemyofibril,...
            'myofibril_decay_num_myofibril', p.myofibrildecaynummyofibril,...
            'myofibril_decay_total_volume', p.myofibrildecaytotalvolume,...
            'myofibril_decay_fiber_area', p.myofibrildecayfiberarea,...
            'myofibril_split_chance', p.myofibrilsplitchance,...
            'myofibril_split_radius', p.myofibrilsplitradius,...
            'myofibril_max_splits', p.myofibrilmaxsplits,...
            'satellite_random_spread', p.satelliterandomspread,...
            'satellite_muscle_pull_spread', p.satellitemusclepullspread,...
            'satellite_quiescent_not', p.satellitequiescentnot,...
            'satellite_quiescent_fused', p.satellitequiescentfused,...
            'satellite_quiescent_num_nuclei', p.satellitequiescentnumnuclei,...
            'satellite_quiescent_fiber_decay', p.satellitequiescentfiberdecay,...
            'steady_kill_satellite_cells', p.steadykillsatellitecells,...
            'steady_kill_myoblasts', p.steadykillmyoblasts,...
            'steady_kill_muscle_fibers', p.steadykillmusclefibers,...
            'steady_kill_myofibrils', p.steadykillmyofibrils,...
            'steady_kill_nuclei', p.steadykillnuclei,...
            'steady_induce_hypertrophy', p.steadyinducehypertrophy,...
            'steady_induce_atrophy', p.steadyinduceatrophy,...
            'hypertrophy_growth_rate_multiplier', p.hypertrophygrowthratemultiplier,...
            'hypertrophy_decay_rate_multiplier', p.hypertrophydecayratemultiplier,...
            'hypertrophy_max_radius_multiplier', p.hypertrophymaxradiusmultiplier,...
            'hypertrophy_split_radius_multiplier', p.hypertrophysplitradiusmultiplier,...
            'hypertrophy_fiber_splitting', p.hypertrophyfibersplitting,...
            'record_movie', p.recordmovie,...
            'scene_number', p.scenenumber,...
            'should_render', p.shouldrender,...
            'max_ticks', p.maxticks,...
            'number_simulations', p.numbersims,...
            'movie_folder', p.moviefolder);
        end
        
        %{
        runSimulation
        ---------------------------------------------------------------------------
        Description:
            Creates, runs, and saves a number of simulations using parallel
            processing
        Inputs:
            app - a simSetup instance
            p - a structure containing simulation parameters
        Outputs:
            NA
        %}
        function runSimulation(app,p,hObject,eventdata)
        app.savedSim = [];
        app.simulation = app.resetSimulation(p);
        for q = 1:app.simulation.numberSimulations
            simmy{q} = app.resetSimulation(p);
        end
        simArray = [];        
        parfor i = 1:app.simulation.numberSimulations    
            rng('default');
            currTick = 0;
            currSim = [];
%             tic;
            for j = 1:simmy{i}.maxTicks
                currTick = currTick + 1;       
                currSim{j} = {simmy{i}.saveState()};
                simmy{i}.tick();                                      
                if (mod(currTick, simmy{i}.tickTime) == 0 && simmy{i}.shouldRender)
                    simmy{i}.render();
                    pause(0.25);
                end
            end
%             toc;
            simArray = [simArray; currSim{:}];
            disp('simulation run');
        end
        simArray;
        app.savedSim = simArray;
            
            
%         Fiber Spreading as of June 27, 2018
%         
%         app.simulation = aSimulationSpace(...
%             'tick_time', 2,...
%             'x_bound', 1,...
%             'y_bound', 1,...
%             'z_bound', 1,...
%             'num_satellite_cells', 2,...
%             'satellite_cell_speed', 0.1,...
%             'myoblast_spawn_chance', 0.25,...
%             'satellite_cell_die_chance', 0.01,...
%             'satellite_cell_radius', 0.02,...
%             'num_myoblasts', 0,...
%             'myoblast_speed', 0.1,...
%             'myoblast_die_chance', 0.0,...
%             'myoblast_radius', 0.2,...
%             'num_nuclei', 0,...
%             'num_muscle_fibers', 0,...
%             'fiber_radius', 0.2,...
%             'fiber_length', 0.5,...
%             'fiber_grow_rate', 0.1,...
%             'fiber_decay_rate', 0.3,...
%             'num_myofibrils', 2,...
%             'myofibril_split_chance', 0.01,...
%             'nucleus_speed', 0.005,...
%             'nucleus_die_chance', 0.001,...
%             'nucleus_radius', 0.025,...
%             'record_movie', false,...
%             'scene_number', 0,...
%             'movie_folder', 'C:\Lab\Chase\Satellite_Cell_Modeling\movietestthrowaway\');
        
        
%         Look at Nuclei, they don't leave the fiber now as of June 27,
%         2018
%         
%         app.simulation = aSimulationSpace(...
%             'tick_time', 1,...
%             'x_bound', 1,...
%             'y_bound', 1,...
%             'z_bound', 1,...
%             'num_satellite_cells', 1,...
%             'satellite_cell_speed', 0.1,...
%             'myoblast_spawn_chance', 0.25,...
%             'satellite_cell_die_chance', 0.01,...
%             'satellite_cell_radius', 0.02,...
%             'num_myoblasts', 0,...
%             'myoblast_speed', 0.1,...
%             'myoblast_die_chance', 0.0,...
%             'myoblast_radius', 0.2,...
%             'num_nuclei', 0,...
%             'num_muscle_fibers', 0,...
%             'fiber_radius', 0.2,...
%             'fiber_length', 0.5,...
%             'fiber_grow_rate', 0.1,...
%             'fiber_decay_rate', 0.3,...
%             'num_myofibrils', 2,...
%             'myofibril_split_chance', 0.01,...
%             'nucleus_speed', 0.005,...
%             'nucleus_die_chance', 0.001,...
%             'nucleus_radius', 0.025,...
%             'record_movie', false,...
%             'scene_number', 0,...
%             'movie_folder', 'C:\Lab\Chase\Satellite_Cell_Modeling\movietestthrowaway\');
        
%         Good Video Setup as of June 27, 2018
% 
%         app.simulation = aSimulationSpace(...
%             'tick_time', 1,...
%             'x_bound', 2,...
%             'y_bound', 2,...
%             'z_bound', 5,...
%             'num_satellite_cells', 10,...
%             'satellite_cell_speed', 0.1,...
%             'myoblast_spawn_chance', 0.05,...
%             'satellite_cell_die_chance', 0.01,...
%             'satellite_cell_radius', 0.02,...
%             'num_myoblasts', 0,...
%             'myoblast_speed', 0.1,...
%             'myoblast_die_chance', 0.0,...
%             'myoblast_radius', 0.2,...
%             'num_nuclei', 0,...
%             'num_muscle_fibers', 0,...
%             'fiber_radius', 0.2,...
%             'fiber_length', 0.5,...
%             'fiber_grow_rate', 0.1,...
%             'fiber_decay_rate', 0.3,...
%             'num_myofibrils', 2,...
%             'myofibril_split_chance', 0.01,...
%             'nucleus_speed', 0.005,...
%             'nucleus_die_chance', 0.001,...
%             'nucleus_radius', 0.025,...
%             'record_movie', false,...
%             'scene_number', 0,...
%             'movie_folder', 'C:\Lab\Chase\Satellite_Cell_Modeling\movietestthrowaway\');
        
        
        
%             app.simulation = aSimulationSpace(...
%             'tick_time', 1,...
%             'x_bound', 5,...
%             'y_bound', 5,...
%             'z_bound', 50,...
%             'num_satellite_cells', 10,...
%             'satellite_cell_speed', 0.1,...
%             'myoblast_spawn_chance', 0.05,...
%             'satellite_cell_die_chance', 0.01,...
%             'satellite_cell_radius', 0.2,...
%             'num_myoblasts', 10,...
%             'myoblast_speed', 0.1,...
%             'myoblast_die_chance', 0.0,...
%             'myoblast_radius', 0.2,...
%             'num_nuclei', 0,...
%             'num_muscle_fibers', 0,...
%             'fiber_radius', 1,...
%             'fiber_length', 5,...
%             'fiber_grow_rate', 0.01,...
%             'fiber_decay_rate', 0.3,...
%             'num_myofibrils', 15,...
%             'nucleus_speed', 0.05,...
%             'nucleus_die_chance', 0.001,...
%             'nucleus_radius', 0.1,...
%             'scene_number', 0,...
%             'movie_folder', 'C:\Users\Campbell\Documents\Chase\Satellite_Cell_Modeling\movietest\');
            
        
            
        end        
    end
end

function simar = saveStuff(simulation,row,col)
    simar{row,col} = simulation.saveState();
end